import base64
import hashlib
from Crypto.PublicKey import RSA
from Crypto.Cipher import AES, PKCS1_OAEP
from Crypto.Util.Padding import pad, unpad
from Crypto.Hash import SHA256
from Crypto.Signature import pss
from Crypto import Random


def generate_rsa_key(bits: int):
    key = RSA.generate(bits)
    private_key = key.export_key()
    public_key = key.publickey().export_key()
    return public_key.decode(), private_key.decode()


def rsa_encrypt(public_key: str, text: bytes):
    """
    Encrypt aes key with the public RSA key
    :param public_key:
    :param text:
    :return:
    """
    public_key = RSA.import_key(public_key)
    cipher = PKCS1_OAEP.new(key=public_key, hashAlgo=SHA256, mgfunc=lambda x, y: pss.MGF1(x, y, SHA256))
    ciphertext = cipher.encrypt(text)
    return ciphertext


def rsa_decrypt(private_key: str, text: bytes):
    """
    Decrypt ase key with the private RSA key
    :param private_key:
    :param text:
    :return:
    """
    private_key = RSA.import_key(private_key)
    cipher = PKCS1_OAEP.new(key=private_key, hashAlgo=SHA256, mgfunc=lambda x, y: pss.MGF1(x, y, SHA256))
    ciphertext = cipher.decrypt(text)
    return ciphertext


class AESCipher:

    def __init__(self, key: bytes, iv: bytes):
        # self.bs = AES.block_size
        self.iv = iv
        self.key = key

    def encrypt(self, raw: str):
        # raw = self._pad(raw)
        # iv = Random.new().read(AES.block_size)
        cipher = AES.new(self.key, AES.MODE_CBC, self.iv)
        return cipher.encrypt(pad(raw.encode(), AES.block_size, style="pkcs7"))
        # return cipher.encrypt(pad(bytearray(raw, 'utf-8'), AES.block_size))
        # return iv + cipher.encrypt(raw.encode())

    def decrypt(self, enc: bytes):
        # iv = enc[:AES.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, self.iv)
        return unpad(cipher.decrypt(enc), AES.block_size).decode()
        # return AESCipher._unpad(cipher.decrypt(enc[AES.block_size:])).decode('utf-8')


if __name__ == "__main__":
    # generate aes, iv, get sender, receiver rsa public key
    # aes_key = str_gen(size=32).encode()
    # _IV = str_gen(size=16).encode()
    aes_key = hashlib.sha256("this is aes key".encode()).digest()
    _IV = Random.new().read(AES.block_size)
    # generate sender rsa keys
    sender_rsa_pub, sender_rsa_pri = generate_rsa_key(2048)

    # encrypt sender msg by aes and iv
    AES_CRYPTO = AESCipher(aes_key, _IV)
    enc_text_by_aes_iv = AES_CRYPTO.encrypt("hi this is test message")
    # encrypt sender msg by base64 and then to str
    en_b64_sender_msg_str = base64.b64encode(enc_text_by_aes_iv).decode()
    # encrypt sender iv by base64 and then to str
    en_b64_sender_iv_str = base64.b64encode(_IV).decode()
    # encrypt sender aes by receiver RSA public key
    enc_aes_by_rsa_pub = rsa_encrypt(open('../ignoreFolder/trial/digital_envelope/receiver/public.bin').read(), aes_key)
    # encrypt sender aes with receiver RSA public key by base64 encode and then str
    en_b64_sender_aes_str = base64.b64encode(enc_aes_by_rsa_pub).decode()
    # combine b64_sender_msg % b64_sender_aes % b64_sender_iv
    request_msg = en_b64_sender_msg_str+"%"+en_b64_sender_aes_str+"%"+en_b64_sender_iv_str

    # receiver split msg by %
    receiver_msg = request_msg.split("%")
    # decrypt sender aes with receiver RSA public key by base64 decode
    de_b64_aes = base64.b64decode(receiver_msg[1])
    # decrypt sender aes by receiver RSA private key
    dec_aes_by_rsa_pri = rsa_decrypt(open("../ignoreFolder/trial/digital_envelope/receiver/private.bin").read(), de_b64_aes)
    # decrypt sender iv by base64 decode
    de_b64_iv = base64.b64decode(receiver_msg[2])
    # decrypt sender msg by base64 decode
    de_b64_msg = base64.b64decode(receiver_msg[0])
    # decrypt sender msg by aes and iv
    dec_text_by_aes_iv = AES_CRYPTO.decrypt(de_b64_msg)
    print(dec_text_by_aes_iv)




import requests
from requests.models import Response
import json as JSON
import re
from types import ModuleType
import logging
import inspect
import warnings


class Request:

    def __init__(self, env_info: ModuleType, test_env: str = None):
        """
        Build a Request object
        :param env_info: env_info.py which is record api related info
        :param test_env: if you want to use specific test env in this instance
        """
        warnings.filterwarnings('ignore', message='Unverified HTTPS request')
        self.session = requests.sessions.Session()
        self.env_info = env_info
        self.url = None
        self.test_config = getattr(env_info, test_env).CONFIG if test_env else getattr(env_info, env_info.TestEnv).CONFIG
        self.logger = logging.getLogger(__name__)

    def re_param(self, url, **kwargs):
        """
        please add own data in env_info.py before used
        """
        if "http" not in url:
            self.url = self.env_info.BuildRequestProperties.build_url(url)
        else:
            self.url = url

        if "headers" not in kwargs:
            self.session.headers = self.env_info.BuildRequestProperties.build_header(url)

    def _custom_logger(self, response):
        def repl(matchobj):
            # print(matchobj.group(0))
            if matchobj.group(0) == "\\":
                return ""
            else:
                return matchobj.group(0)[:-1] + "."
        caller_frame = inspect.currentframe().f_back.f_back
        frame_info = inspect.getframeinfo(caller_frame)
        file_path = re.search(rf"{re.escape(str(self.env_info.project_root_dir))}(.+)", frame_info.filename).group(1)
        function_name = frame_info.function
        line_number = frame_info.lineno

        matched_regex = ".*?\\\\"
        response_log = self.format_res_log(response)
        self.logger.debug(
            f'{re.sub(matched_regex, repl, file_path)}:{function_name}[{line_number}] {JSON.dumps(response_log)}',
            extra=response_log
        )

    def get(self, url, logged=True, **kwargs):
        self.re_param(url, **kwargs)
        res = self.session.get(self.url, **kwargs, verify=False if 'https://' in self.url else None)
        if logged is True:
            self._custom_logger(res)
        return res

    def post(self, url, data=None, json=None, logged=True, **kwargs):
        self.re_param(url, **kwargs)
        res = self.session.post(self.url, data, json, **kwargs, verify=False if 'https://' in self.url else None)
        if logged is True:
            self._custom_logger(res)
        return res

    def put(self, url, data=None, **kwargs):
        self.re_param(url, **kwargs)
        res = self.session.put(self.url, data, **kwargs, verify=False if 'https://' in self.url else None)
        self._custom_logger(res)
        return res

    def delete(self, url, **kwargs):
        self.re_param(url, **kwargs)
        res = self.session.delete(self.url, **kwargs, verify=False if 'https://' in self.url else None)
        self._custom_logger(res)
        return res

    @staticmethod
    def format_res_log(res: Response) -> dict:
        """
        Change Response Obj to log format

        :param res: Response Obj
        :return: log format as dict
        """
        if res.request.method == "GET":
            return {
                "request": {
                    "url": str(res.request.url),
                    "headers": dict(res.request.headers)
                },
                "response": {
                    "headers": dict(res.headers),
                    "body": res.json()
                }
            } if re.search(r"json", res.headers["content-type"]) else {
                "request": {
                    "url": str(res.request.url),
                    "headers": dict(res.request.headers)
                },
                "response": {
                    "headers": dict(res.headers)
                }
            }
        elif res.request.method == "POST":
            return {
                "request": {
                    "url": str(res.request.url),
                    "headers": dict(res.request.headers),
                    "body": JSON.loads(res.request.body) if res.request.headers.get("content-type") == "application/json" else None
                },
                "response": {
                    "headers": dict(res.headers),
                    "body": res.json() if res.headers.get("content-type") and "json" in res.headers.get("content-type") else None
                }
            }
        elif res.request.method == "PUT":
            return {
                "request": {
                    "url": str(res.request.url),
                    "headers": dict(res.request.headers),
                    "body": JSON.loads(res.request.body)
                },
                "response": {
                    "headers": dict(res.headers),
                    "body": res.json() if res.headers.get("content-type") and "json" in res.headers.get("content-type") else None
                }
            }
        elif res.request.method == "DELETE":
            return {
                "request": {
                    "url": str(res.request.url),
                    "headers": dict(res.request.headers)
                },
                "response": {
                    "headers": dict(res.headers),
                    "body": res.json() if res.headers.get("content-type") and "json" in res.headers.get("content-type") else None
                }
            }




from threading import Timer
from functools import reduce
import operator
import jsonschema
import random
import string


def json_schema_validate(data: dict, schema: dict, index=""):
    """
    Validate an instance data under the given schema

    :param data: response json as dict
    :param schema: api json schema as dict
    :param index: input data index for trace used, example: userId
    """
    validator = jsonschema.Draft202012Validator(schema, format_checker=jsonschema.FormatChecker())

    errors = validator.iter_errors(data)
    schema_errors = ""
    for error in errors:
        schema_errors += f"\n{error.absolute_path}, {error.message}, {index}"
    if schema_errors:
        # raise jsonschema.ValidationError(schema_errors)
        raise AssertionError(schema_errors)


def find_column(sheet, index_row, find_str):
    for cell in sheet[str(index_row)]:
        if find_str == cell.value:
            return cell


def str_gen(char=string.ascii_letters + string.digits, size=20):
    return ''.join(random.choices(char, k=size))


class RepeatingTimer(Timer):
    def run(self):
        self.finished.wait(self.interval)
        while not self.finished.is_set():
            self.function(*self.args, **self.kwargs)
            self.finished.wait(self.interval)


class DictTool:
    @staticmethod
    def transfer_json_path(path: str):
        """
        transfer json path to list
        :param path: json path, ex: "TRANRS[0].district[0].districtName"
        :return: list of json path , ex: ['TRANRS', 0, 'district', 0, 'districtName']
        """
        return [int(s) if s.isdigit() else s for s in re.split(r"\.", re.sub(r"\[(\d+)]", r".\g<1>", path))]

    @classmethod
    def get_by_path(cls, root: dict, items: list):
        """
        Access a nested object in root by item sequence.
        :param root: api request payload as dict format
        :param items: key path, ex: ['TRANRS', 0, 'district', 0, 'districtName']
        :return:
        """
        return reduce(operator.getitem, items, root)

    @classmethod
    def set_by_path(cls, root: dict, items: list, value):
        """
        Set a value in a nested object in root by item sequence.
        :param root: api request payload as dict format
        :param items: key path, ex: ['TRANRS', 0, 'district', 0, 'districtName']
        :param value: value of key that you want to set
        :return:
        """
        cls.get_by_path(root, items[:-1])[items[-1]] = value



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cdc-automation",
    version="0.0.2",
    author="Wayne Chen",
    author_email="kensmart123@yahoo.com.tw",
    description="only for cdc own study used",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'requests',
        'python-json-logger',
        'jsonschema',
        'webcolors',  # jsonschema validating formats
        'rfc3339-validator',  # jsonschema validating formats
        'isoduration',  # jsonschema validating formats
        'fqdn',  # jsonschema validating formats
        'idna',  # jsonschema validating formats
        'rfc3987',  # jsonschema validating formats
        'jsonpointer',  # jsonschema validating formats
        'uri-template',  # jsonschema validating formats
        'pycryptodome',  # handle AES, RSA crypto
    ]
)